/*
 * pc300util.h	Cyclades-PC300(tm) debug utility.
 *
 * Author:	Daniela Squassoni <daniela@cyclades.com>
 *
 * Copyright:	(c) 1999-2001 Cyclades Corp.
 *
 *      This program is free software; you can redistribute it and/or
 *      modify it under the terms of the GNU General Public License
 *      as published by the Free Software Foundation; either version
 *      2 of the License, or (at your option) any later version.
 *
 * $Log: pc300.h,v $
 * Revision 0.2 2001/03/02 Daniela 
 * Implemented -H option.
 *
 * Revision 0.1 2000/12/14 Daniela 
 * Initial version.
 *
 */

#include "pc300.h"

/***************/
/*  Constants  */
/***************/

#define DO_CONFIG	0x0001	/* Show the board configuration */
#define DO_STATISTICS   0x0002	/* Show statistics */
#define DO_STATUS	0x0004	/* Show link status */
#define DO_LOCLOOP 	0x0008	/* Set the interface to local loopback */
#define DO_REMLOOP 	0x0010	/* Set the interface to remote loopback */
#define DO_PAYLOOP	0x0020	/* Set Payload loop */ 
#define DO_LOOPUP	0x0040	/* Generate loop UP */
#define DO_LOOPDOWN	0x0080	/* Generate loop DOWN */
#define DO_PATTERN	0x0100	/* Set Pattern test */
#define DO_TRACE	0x0200	/* Start the line trace */
#define DO_FILE		0x0400	/* Select the name of the file for trace */
#define DO_HELP		0x0800	/* Show the help text */
#define DO_HARDWARE	0x1000	/* Show hardware parameters */

#define MASK_OPENSOCK	0x11FE	/* Options that request a socket call */

/**********************/
/*  Global Variables  */
/**********************/

char progname[] = "pc300util";
char banner[] =	"Cyclades pc300util v1.0.0 "
				"(c) 2000 Cyclades Corporation";

unsigned char *pversions = "@(#) pc300util : v_1.0.0	12/14/00";

char helptext[] = "Usage:"
                "\tpc300util [-csStlrpUDP] [-d device] [-f trace_file]\n\nDetailed help:\tpc300util -h\n";

char helptext_detailed[] = "
# pc300util [-csStlrpUDPHh] [-d hdlcN] [-f file_name]
-c: Show the board configuration.
-s: Show statistics.
-S: Show link status 
        - RSV/X.21 boards: DCD, DSR, CTS, RTS, DTR.
        - TE boards: synchronized/ not synchronized, link alarms.
-t: Starts the line trace; continues until <CTRL+C> is pressed.
-l: For TE boards only; set the interface to local loopback. 
-r: For TE boards only; set the interface to remote loopback. 
-p: For TE boards only; set the interface to payload loop. 
-U: For TE boards only; generate loop UP signal, indicating that the remote
    interface should set itself to loop.
-D: For TE boards only; generate loop DOWN signal, indicating that the remote
    interface should stop looping. 
-P: For TE boards only; Pattern test. 
-H: Show the PC300 hardware information.
-h: Help.
-d: Selects the device name (hdlcN). If the name of the interface is not 
    supplied, hdlc0 will be assumed.
-f: Defines the file to wich the trace, statistics or status information will 
    be written. If not included, the information will appear only on the screen.

Examples: 
# pc300util -c -d hdlc0 
# pc300util -t -d hdlc1 -f trc_hdlc1 
# pc300util -csS -d hdlc2 ";

char pc300cfg_string[128] = "/usr/local/sbin/pc300cfg ";

char device [6] = "hdlc0";
int options_mask = 0;
char gl_loop_type = 0;
char gl_patterntst_on = 0;
int skfd = -1;
int s = -1;

char trace_on;
char *pc300_file;
char *program_name;
extern int optind;
extern int opterr;
extern char *optarg;

/*****************************/
/*       Prototypes	     */
/*****************************/

void hex_dump(unsigned char *, int);
void exec_action(int);
void pc300util_end(int);

#define RETSIGTYPE void

RETSIGTYPE cleanup(int);
RETSIGTYPE (*setsignal(int, RETSIGTYPE (*)(int)))(int);
