/*****************************************************************************
* chdlc_rec.c	CHDLC API: Receive Module
*
* Author(s):	Gideon Hack & Nenad Corbic <ncorbic@sangoma.com>
*
* Copyright:	(c) 1995-2000 Sangoma Technologies Inc.
*
*		This program is free software; you can redistribute it and/or
*		modify it under the terms of the GNU General Public License
*		as published by the Free Software Foundation; either version
*		2 of the License, or (at your option) any later version.
* ============================================================================
*/


#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <linux/if_wanpipe.h>
#include <string.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <linux/wanpipe.h>
#include <linux/sdla_chdlc.h>

#define FALSE	0
#define TRUE	1

#define LGTH_CRC_BYTES	2
#define MAX_TX_DATA     1000	/* Size of tx data */  
#define MAX_FRAMES 	5000     /* Number of frames to transmit */  

#define WRITE 1
unsigned short no_CRC_bytes_Rx;
unsigned char HDLC_streaming = FALSE;
unsigned short Rx_lgth;

unsigned char Rx_data[16000];
unsigned char Tx_data[MAX_TX_DATA + sizeof(api_tx_hdr_t)];


/* Prototypes */
int MakeConnection(char *, char *);
void recv_socket( void);

int 	sock;
/***************************************************
* MakeConnection
*
*   o Create a Socket
*   o Bind a socket to a wanpipe network interface
*       (Interface name is supplied by the user)
*/         

int MakeConnection(char *r_name, char *i_name ) 
{
	struct wan_sockaddr_ll 	sa;

	memset(&sa,0,sizeof(struct wan_sockaddr_ll));
	errno = 0;
   	sock = socket(AF_WANPIPE, SOCK_RAW, 0);
   	if( sock < 0 ) {
      		perror("Socket");
      		return( FALSE );
   	} /* if */
  
	printf("\nConnecting to router %s, interface %s\n", r_name, i_name);
	
	strcpy( sa.sll_device, i_name);
	strcpy( sa.sll_card, r_name);
	sa.sll_protocol = htons(PVC_PROT);
	sa.sll_family=AF_WANPIPE;

        if(bind(sock, (struct sockaddr *)&sa, sizeof(struct wan_sockaddr_ll)) < 0){
                perror("bind");
		printf("Failed to bind a socket to %s interface\n",i_name);
                exit(0);
        }
	printf("Socket bound to %s\n\n",i_name);

	return( TRUE );

}

/***************************************************
* RECV_SOCKET 
*
*   o Read a socket 
*   o Cast data received to api_rx_element_t data type 
*   o The received packet contains 16 bytes header 
*
*	------------------------------------------
*      |  16 bytes      |        X bytes        ...
*	------------------------------------------
* 	   Header              Data
*
*   o Data structures:
*
*	typedef struct {
*        	unsigned char   error_flag      PACKED;
*        	unsigned short  time_stamp      PACKED;
*        	unsigned char   reserved[13]    PACKED;
*	} api_rx_hdr_t; 
*
*	typedef struct {
*        	api_rx_hdr_t    api_rx_hdr      PACKED;
*        	void *          data            PACKED;
*	} api_rx_element_t;
*
*
*/

void recv_socket(void) 
{
	unsigned int Rx_count,Tx_count,Tx_length;
	//api_rx_element_t* api_rx_el;
	api_tx_element_t * api_tx_el;
	fd_set 	ready,write;
	int err,i;
	unsigned char *data;

        Rx_count = 0;
	Tx_count = 0;
	Tx_length = MAX_TX_DATA;
	
	/* If running HDLC_STREAMING then the received CRC bytes
         * will be passed to the application as part of the 
         * received data.  
	 */

	memset(&Tx_data[0],0,MAX_TX_DATA + sizeof(api_tx_hdr_t));

	api_tx_el = (api_tx_element_t*)&Tx_data[0];
	data = (unsigned char *)&api_tx_el->data;

	for (i=0;i<Tx_length;i++){
		data[i] = (unsigned char)i;
	}
	

	no_CRC_bytes_Rx = 0; //LGTH_CRC_BYTES;

	for(;;) {	
		FD_ZERO(&ready);
		FD_ZERO(&write);
		FD_SET(sock,&ready);
#ifdef WRITE
		FD_SET(sock,&write);
#endif

  		if(select(sock + 1,&ready, &write, NULL, NULL)){


                   if (FD_ISSET(sock,&ready)){

			err = recv(sock, Rx_data, sizeof(Rx_data), 0);

			/* err indicates bytes received */
			if(err != -1) {

		                /* Check the packet length */
                                Rx_lgth = err - sizeof(api_rx_hdr_t);
                                if(Rx_lgth <= no_CRC_bytes_Rx) {
                                        printf("\nShort frame received (%d)\n",
                                                Rx_lgth);
                                        return;
                                }

                                Rx_lgth -= no_CRC_bytes_Rx;

				if(!(Rx_count % 1)) {	
					printf("Received packet %i, length = %i\n", 
						++Rx_count, Rx_lgth);
				}
			} else {
				printf("\nError receiving data\n");
				break;
			}

		   }else if (FD_ISSET(sock,&write)){

			err = send(sock,Tx_data, Tx_length + sizeof(api_tx_hdr_t), 0);

			if (err <= 0){

				printf("Faild to send\n");
				break;
			}else{
				printf("Packet sent: Sent %i : %i\n",
						err,++Tx_count);
			}
		   }

			if (Tx_count == MAX_FRAMES)
				break;

			// sleep(0.5);
		} else {
                	printf("\nError selecting socket\n");
			break;
		}
	}
	while (ioctl(sock,SIOC_WANPIPE_CHECK_TX,0));
	close (sock);
} 



/***************************************************************
 * Main:
 *
 *    o Make a socket connection to the driver.
 *    o Call recv_socket() to read the socket 
 *
 **************************************************************/


int main(int argc, char* argv[])
{
	int proceed;

	if (argc != 3){
		printf("Usage: rec_wan_sock <router name> <interface name>\n");
		exit(0);
	}	

	proceed = MakeConnection(argv[argc - 2], argv[argc - 1]);
	if( proceed == TRUE ){
		recv_socket( );
		return 0;
	}

	return 1;
};
