/*****************************************************************************
* fr_api.c	CHDLC API: Transmit Module
*
* Author(s):	Gideon Hack & Nenad Corbic <ncorbic@sangoma.com>
*
* Copyright:	(c) 1995-2000 Sangoma Technologies Inc.
*
*		This program is free software; you can redistribute it and/or
*		modify it under the terms of the GNU General Public License
*		as published by the Free Software Foundation; either version
*		2 of the License, or (at your option) any later version.
* ============================================================================
*/

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/time.h>
#include <sys/socket.h>
//#include <linux/socket.h>
#include <sys/ioctl.h>
#include <signal.h>
#include <time.h>
#include <netinet/in.h>
#include <linux/if_wanpipe.h>
#include <string.h>
#include <errno.h>
#include <linux/wanpipe.h>
#include <linux/sdla_fr.h>
#include <linux/if_ether.h>

#define FALSE	0
#define TRUE	1

#define MIN_FRM_LGTH	100	
#define MAX_FRM_LGTH  	1000
#define NO_FRMS_TO_TX   10000	

#define WRITE	1
 
unsigned char Tx_data[MAX_FRM_LGTH + sizeof(api_tx_hdr_t)];
unsigned char Rx_data[MAX_FRM_LGTH + sizeof(api_tx_hdr_t)];

/* Prototypes */
int MakeConnection(char *,  char *);
void send_socket( void);
void setup_signal_handlers (void);
void quit (int);
void sig_urg (int);

int sock;
fd_set writeset,readset,oobset;
struct wan_sockaddr_ll sa;


/***************************************************************
 * Main:
 *
 *    o Make a socket connection to the driver.
 *    o Call send_socket() to transmit data
 *
 **************************************************************/   

int main(int argc, char* argv[])
{
	int proceed;
	
	if (argc != 3){
                printf("Usage: fr_send <router name> <interface name>\n");
                exit(0);
        }

        proceed = MakeConnection(argv[argc - 2], argv[argc - 1]);

	if( proceed == TRUE ){
		printf("Sending a command\n");
		send_socket( );
		printf("End of command\n");
		return 0;
	}
	return 1;
};



/***************************************************
* MakeConnection
*
*   o Create a Socket
*   o Bind a socket to a wanpipe network interface
*       (Interface name is supplied by the user)
*/         

int MakeConnection (char *r_name, char *i_name) 
{

	errno = 0;
   	sock = socket( AF_WANPIPE, SOCK_RAW, 0);
   	if( sock < 0 ) {
      		perror("Socket");
      		return( FALSE );
   	} /* if */
 
	printf("Socket Created OK\n"); 
	sa.sll_family = AF_WANPIPE;
	sa.sll_protocol = htons(ETH_P_IP);
	strcpy(sa.sll_device, i_name);
        strcpy( sa.sll_card, r_name);
	sa.sll_ifindex = 0;

        if(bind(sock, (struct sockaddr *)&sa, sizeof(struct wan_sockaddr_ll)) < 0){
                perror("bind");
		printf("Failed to bind socket to %s interface\n",i_name);
                exit(0);
        }
	printf("Socket bound to %s\n",i_name);
	fflush(stdout);
	return( TRUE );
}

/***************************************************
* SEND_SOCKET 
*
*   o Send data throught a socket 
*   o Create a packet using api_tx_element_t data type. 
*   o The transmit packet contains 16 bytes header 
*
*	------------------------------------------
*      |  16 bytes      |        X bytes        ...
*	------------------------------------------
* 	   Header              Data
*
*   o Data structures:
*
* 	typedef struct {
*        	unsigned char   attr            PACKED;
*      	 	unsigned char   reserved[15]    PACKED;
*	} api_tx_hdr_t;
*
*	typedef struct {
*        	api_tx_hdr_t    api_tx_hdr      PACKED;
*        	void *          data            PACKED;
*	} api_tx_element_t;
*/

void send_socket(void)
{
	unsigned int Tx_count=0,Rx_count=0;
	unsigned short Tx_length=0,Rx_length=0;
	api_tx_element_t* api_tx_el;
	int err=0, i=0;

	Tx_length = MIN_FRM_LGTH;
	Rx_length = MAX_FRM_LGTH;


	api_tx_el = (api_tx_element_t *)Tx_data;
  
	/* Fill in the packet data */ 

	for (i=0; i<Tx_length ; i++){
               	*((unsigned char *)&api_tx_el->data + i) = (unsigned char) i;
	}

	for(;;) {
		FD_ZERO(&writeset);
		FD_ZERO(&readset);
		FD_ZERO(&oobset);

#ifdef WRITE
		FD_SET(sock,&writeset);
#endif
		FD_SET(sock,&readset);
		FD_SET(sock,&oobset);

	        if(select(sock + 1, &readset, &writeset, &oobset, NULL)){


			if (FD_ISSET(sock,&writeset)){

				err = send(sock, Tx_data, Tx_length + sizeof(api_tx_hdr_t), 0);

				/* err contains number of bytes transmited */		
				if(err < 0) {
					printf("Failed to send %i , %i\n", Tx_count, err);
					exit(1);
				}else{
	                                printf("Packet Sent %i, length = %i\n",
						++Tx_count, Tx_length);
				}
			}else if (FD_ISSET(sock,&readset)){

				err = recv(sock,Rx_data, Rx_length + sizeof(api_rx_hdr_t),0);
				
				if (err < 0){
					printf("Failed to rec %i , %i\n", Rx_count, err);
					exit(1);
				}else{
					printf("Packet receive %i, length = %i\n",
						++Rx_count, err);
				}	
			}
	
			/* Stop after number of frames */
			if(Tx_count == NO_FRMS_TO_TX)
				break;

		}
	}

	/* Make sure all data has gone out the port before
         * closing the socket.  If a socket is closed before
         * all packets are transmitted, data will be lost */
         
	while (ioctl(sock,SIOC_WANPIPE_CHECK_TX,0));
	close(sock);
} 
