/*****************************************************************************
* nowait_client.c  X25 API: SVC Client Application : NO WAIT MODE
*
* Author(s):	Nenad Corbic <ncorbic@sangoma.com>
*
* Copyright:	(c) 1995-2000 Sangoma Technologies Inc.
*
*		This program is free software; you can redistribute it and/or
*		modify it under the terms of the GNU General Public License
*		as published by the Free Software Foundation; either version
*		2 of the License, or (at your option) any later version.
* ============================================================================
*/

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/time.h>
#include <sys/socket.h>
//#include <linux/socket.h>
#include <sys/ioctl.h>
#include <signal.h>
#include <time.h>
#include <netinet/in.h>
#include <linux/if_wanpipe.h>
#include <string.h>
#include <errno.h>
#include <linux/wanpipe.h>
#include <linux/sdla_x25.h>
#include <linux/if_ether.h>

#define FALSE	0
#define TRUE	1

#define MIN_FRM_LGTH	100	
#define MAX_FRM_LGTH  	1400
#define NO_FRMS_TO_TX   10	

#define MAX_X25_ADDR_SIZE	16
#define MAX_X25_DATA_SIZE 	129
#define MAX_X25_FACL_SIZE	110


#define TIMEOUT 10
unsigned char Tx_data[MIN_FRM_LGTH + sizeof(x25api_hdr_t)];

unsigned char Rx_data[1000];

/* Prototypes */
int MakeConnection( char * );
void send_socket( void);


/* Defined global so it would only get allocated once.  
 * If it was defined in MakeConnection() function it 
 * would be allocated and dealocated every time we 
 * run that function.
 */  

int sock, sock1;
x25api_t api_cmd;


/***************************************************************
 * Main:
 *
 *    o Make a socket connection to the driver.
 *    o Call send_socket() to transmit data
 *
 **************************************************************/   

int main(int argc, char* argv[])
{
	int proceed;
	
	if (argc != 2){
		printf("Usage: ./nowait_client <card name>\n");
		exit(0);
	}

again:
	proceed = MakeConnection(argv[argc-1]);
	if (proceed == 0){
		goto again;
	}else{
		send_socket( );
		goto again;
		return 0;
	}
	return 1;
};






/***************************************************
* MakeConnection
*
*   o Create a Socket
*   o Bind a socket to a wanpipe network interface
*/         

int MakeConnection (char *i_name ) 
{
	int len = sizeof(struct wan_sockaddr_ll);
	int err = 0;
	struct wan_sockaddr_ll 	sa;

	memset(&sa,0,sizeof(struct wan_sockaddr_ll));

	/* Create a new socket */
   	sock = socket( AF_WANPIPE, SOCK_RAW, 0);
   	if( sock < 0 ) {
      		perror("Socket");
      		return 1;
   	}

	/* Fill in binding information 
	 * before we use connect() system call
         * a socket must be binded into a virtual
         * network interface svc_connect.
         * 
         * Card name must be supplied as well. In 
         * this case the user supplies the name
         * eg: wanpipe1.
         */ 
	sa.sll_family = AF_WANPIPE;
	sa.sll_protocol = htons(X25_PROT);
	strcpy(sa.sll_device, "svc_connect");
	strcpy(sa.sll_card, i_name);
	
	/* Bind a sock using the above address structure */
        if(bind(sock, (struct sockaddr *)&sa, sizeof(struct wan_sockaddr_ll)) < 0){
                perror("bind");
		printf("Failed to bind socket to %s interface\n",i_name);
		close(sock);
                exit(0);
        }

	/* Reset the api command structure */
	memset(&api_cmd, 0, sizeof(x25api_t));

	/* Write the call data into the command structure.
         * Note: -u data must be have even number of characters.
         *       -f field cannot be set to zero (i.e. -f0) 
         *       Refer to x25.pdf hardware manual.
         */
	sprintf(api_cmd.data,"-d4172625101970 -s41101190 -uC700000054999400");
	api_cmd.hdr.length = strlen(api_cmd.data);

	/* Write the call data into a socket mail box using
         * the following ioctl call.  This must be one before
         * the connect system call 
         */

	if ((err=ioctl(sock,SIOC_WANPIPE_SET_CALL_DATA,&api_cmd)) < 0){
		printf ("Setting call data Failed\n");
		close(sock);
		exit(0);
	}else{
		printf("Set Call Data OK\n");
	}

	/* In this case we do not want to wait until the 
         * connection is established. By tunning SET NON_BLOCK
         * ioctl call, the connect() system call will not wait 
         * for the remote reply */
	
	if ((err=ioctl(sock,SIOC_WANPIPE_SET_NONBLOCK,0)) < 0){
		printf ("IOCTL SET CALL FAILED\n");
		close(sock);
		exit(1);
	}else{
		printf("Socket set to nonblocking OK\n");
	}

		
	/* Connect will not wait for remote reply, it
         * will return zero indicating that the PLACE CALL 
         * command was successful.
         */ 

	/* The sa address structure is optional. The sa 
         * structure will contain a network interface name 
         * on which the connection was
         * established */
	
	err = connect(sock,(struct sockaddr *)&sa,len);

	if (err == 0){
		printf("Connect OK %i\n",err);
	}else{
		printf("Connect BAD : %i \n",err);
		close(sock);
		return (FALSE);
	}

	/* If we came to this point, we can place repeat the above
         * process for another call, or continue to wait for the
         * connection to come up. */

	fflush(stdout);
	return( TRUE );
}


/***************************************************
* SEND_RECEIVE_SOCKET 
*
*   o Send data throught a socket 
*   o Create a packet using x25api_t data type. 
*   o The transmit packet contains 16 bytes header 
*
*	------------------------------------------
*      |  16 bytes      |        X bytes        ...
*	------------------------------------------
* 	   Header              Data
*
*   o Data structures:
*
*     	typedef struct {
*		unsigned char  qdm	PACKED;	Q/D/M bits 
*		unsigned char  cause	PACKED;	cause field 
*		unsigned char  diagn	PACKED; diagnostics 
*		unsigned char  pktType  PACKED; 
*		unsigned short length   PACKED;
*		unsigned char  result	PACKED;
*		unsigned short lcn	PACKED;
*		char reserved[7]	PACKED;
*	}x25api_hdr_t;
*
*	typedef struct {
*		x25api_hdr_t hdr	PACKED;
*		char data[X25_MAX_DATA]	PACKED;
*	}x25api_t;
*
*
* Note: when sending DATA: driver will only look at the QDM byte of 
*                          the header.
*
* QDM: byte is a bit map of three bits:
*
* 	Q bit: Bit 0 : Qualifier bit, special kind of packet.
*                      Can be used as control packet.  
*       D bit: Bit 1 : Data acknolwedge bit. The remote will
*                      acknowledge every packet sent.
*       M bit: Bit 2 : If your packet is greater than x25 MTU,
*                      i.e 1024 bytes, than cut the packet to MTU size
*                          and set the M bit to indicate more data.                
*/

void send_socket(void)
{
	unsigned int Tx_count=0;
	unsigned short Tx_lgth,timeout=0;
	int err=0, i, Rx_count=0;
	struct timeval tv;
	x25api_t* api_tx_el;
	fd_set writeset,readset,oobset;


	tv.tv_usec = 0;  //10ms
	tv.tv_sec = 5;
	Tx_lgth = MIN_FRM_LGTH;

	api_tx_el = (x25api_t *)Tx_data;
 
	/* Fill in the packet data */ 

	for (i=0; i<Tx_lgth ; i++){
               	api_tx_el->data[i] = (unsigned char) i;
	}

	printf("Waiting for connection\n");

	for(;;) {
		FD_ZERO(&readset);
		FD_ZERO(&oobset);
		FD_ZERO(&writeset);
		
		FD_SET(sock,&readset);
		FD_SET(sock,&oobset);
		FD_SET(sock,&writeset);

		/* This must be within the loop */	
		tv.tv_usec = 0;  
		tv.tv_sec = TIMEOUT;


		/* The select function must be used to implement flow control.
	         * WANPIPE socket will block the user if already queued packet
                 * cannot be sent 
		 */ 

		/* Furthermore, the select function will also wait for
                 * the connection to come up.  If the connection failed
                 * we will get an OOB message indication.  Otherwise,
                 * if the connecton succeded we will be able to send or
                 * receive data. */

		/* Note, we can get the lcn number we are connected on
                 * from any data packet we receive */ 

	        if(select(sock + 1,&readset, &writeset, &oobset, &tv)){

			if (FD_ISSET(sock,&readset)){
				
				err = recv(sock, Rx_data, sizeof(Rx_data), 0);

				/* err contains number of bytes transmited */		
				if(err < 0 ) {
					printf("Failed to rcv %i , %i\n", 
							Rx_count, err);
					close(sock);
					exit(1);
				}else{
					/* Every received packet comes with the 16
					 * bytes of header which driver adds on.
                                         * Header is structured as x25api_hdr_t. 
					 * (same as above)
					 */

					x25api_hdr_t *api_data = 
							(x25api_hdr_t *)Rx_data;
					printf("\tReceive OK, size: %i, qdm %x,"
						 "cause %x, diagn %x, cnt: %i\n", 
						  err, api_data->qdm, 
						  api_data->cause, 
						  api_data->diagn, ++Rx_count);
				}
		
			}else if (FD_ISSET(sock,&writeset)){

				
				err = send(sock, Tx_data, 
					   Tx_lgth + sizeof(x25api_hdr_t), 0);

				/* err contains number of bytes transmited */		
				if(err <0 ) {
					printf("Failed to send %i , %i\n", 
								Tx_count, err);
					close(sock);
					exit(1);
				}else{
					printf("Packet Sent %i\n",++Tx_count);
				}				

			}else if (FD_ISSET(sock,&oobset)){

				printf("READING OOB MSG\n");

				/* The OOB Message will indicate that an 
				 * asynchronous event occured.  The applicaton 
			         * must stop everything and check the state of 
				 * the link.  Since link might  
                                 * have gone down */

				/* In this example I just exit. A real 
				 * application would check the state of the 
			         * sock first by reading the header information. 
				 */

				/* If we fail to read the OOB message, we can 
				 * assume that the link is down.  Thus, close 
				 * the socket and continue */

				err = recv(sock, Rx_data, sizeof(Rx_data), MSG_OOB);
		
				/* err contains number of bytes transmited */		
				if(err < 0 ) {
					printf("Failed to receive OOB %i , %i\n", 
						Rx_count, err);
					break;
				}else{

					api_tx_el = (x25api_t *)Rx_data;

					printf("OOB Packet type %x, Cause %x,"
				   	   " Diagn %x, Res %x, Len %i, LCN %i\n",
						api_tx_el->hdr.pktType, 
						api_tx_el->hdr.cause,
						api_tx_el->hdr.diagn, 
						api_tx_el->hdr.result,
 						api_tx_el->hdr.length,
						api_tx_el->hdr.lcn);
					break;
				}
			}

			/* Stop after number of frames */
			//if(Tx_count == NO_FRMS_TO_TX)
			//	break;
			timeout=0;
		}else{
			if (err == 0){
				if (++timeout == 5){
					printf("Sock timeout exceeded MAXIMUM\n");
					break;			
				}		
				printf("Sock timeout try again !!!\n");
			}else{
				printf("Error in Select !!!\n");
				break;
			}
		}
	}
	close(sock);
	return;
} 
